#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <getopt.h>

/*
   Copyright (C) 2003 Red Hat, Inc. All rights reserved.
                                                                                                             
   Usage and distribution of this file are subject to the Gnu General Public License Version 2 
   that can be found at http://www.gnu.org/licenses/gpl.txt and the COPYING file as
   distributed together with this file is included herein by reference. 
                                                                                                                      
   Authors: Arjan van de Ven   <arjanv@redhat.com>
            Eric Dorland       <eric@debian.org>
*/

#include "prototypes.h"

int irq_cost[MAX_CPU];		/* cummulative assigned irq's already */
int this_type[MAX_CPU];		/* number of irq's already assigned within this class */
unsigned long long int affinity_mask = 0xffffffffffffffffULL; /* affinity mask */
struct irqdescriptor interrupts[MAX_INTERRUPTS];


int rotate_premium = 12500;

void term_signal(int signal)
{
    unlink("/var/run/irqbalance.pid");
    exit(EXIT_SUCCESS);
}

void commandpolicy(char *pol, int phase)
{
	/* Allow --noethernet */
	if (!strcmp("--noethernet",pol)) {
		class_policy[IRQ_ETH] = POLICY_IGNORE;
		class_policy[IRQ_GIGE] = POLICY_IGNORE;
		if (phase>0) {
			class_policy[IRQ_OTHER] = POLICY_ROTATE;
			class_policy[IRQ_LEGACY] = POLICY_ROTATE;
			class_policy[IRQ_SCSI] = POLICY_ROTATE;
			class_policy[IRQ_TIMER] = POLICY_ROTATE;
		}
	}

}


static struct option long_options[] ={
	{"oneshot", 0, 0, 1},
	{"afin_mask", 1, 0, 2},
	{0, 0, 0, 0}
}; 

int main(int argc, char **argv)
{
	FILE *pid_file;
	int oneshot = 0;
	int option_index = 0;
	int c;
	
	class_policy[IRQ_TIMER] = POLICY_ROTATE;
	
	parse_proc_cpuinfo();

	while (1) {
		c = getopt_long (argc, argv, "oa:", long_options, &option_index);

		if (c == -1)
			break;

		switch(c) {
			case 1:
				/* oneshot */
				oneshot = 1;
				break;
			case 2:
				/* affinity mask */
				affinity_mask = strtoull(optarg, NULL, 16);
				break;
		}
	}	
	
	if (argc>1) 		
		commandpolicy(argv[1], 0);
	
	if (cpucount < 2) 
		exit(EXIT_SUCCESS); /* UP balancing is useless */
		
	parse_proc_interrupts(0); 

#ifndef DEBUG
	pid_file = fopen("/var/run/irqbalance.pid", "w");
	if (pid_file == NULL)
		exit(EXIT_FAILURE);
	daemon(0,0);
	signal(SIGTERM, term_signal);
	fprintf(pid_file, "%d\n", getpid());
	fclose(pid_file);
    
#endif
	/* 
	 * Allow 5 seconds for gathering irq counts before the first balance; 60 in case of one-shot
	 * balancing 
	 */
	sleep(5 + 55*oneshot);  
	while (1) {
		parse_proc_interrupts(1); 
		balance_policy();
		activate_irqtable();
		if (argc>1)
			commandpolicy(argv[1], 1);

#ifdef DEBUG
		dump_settings();
#endif
		if (oneshot)
			break;
		sleep(10);
	}
	return EXIT_SUCCESS;
}
